apcglmfit_run <-
function(r, header = F, n.risk=NA, apcmodel="AP", fam = "loglin", Plot = F,  pylim = c(0, 0), 
  Scale=1e-5,	c1=1, c2=1, cc=1, apc=0, lindex=0, lvalue=0, amin = 1, pmin = 1, cmin = 1, n.interval = 1, 
  stderrplot=T, pcex=1) {
### fit APC data with loglinear model, using numerator and denominator.
  if (header == T) {
  age = as.numeric(dimnames(r)[[1]])
  period = as.numeric(dimnames(r)[[2]])
	if (min(period) < 200)
		period = period + 1900
	cmin = period [1] - age[nrow(r)]
	cohort = cmin + c(1:(nrow(r)+ncol(r)-1)-1)*(age[3]-age[2])
 }

a = nrow(r)
p = ncol(r)
x = apcmat(a, p)

#if (Plot == T) {
   if (header == F) {
	age = amin + (c(1:a) - 1) * n.interval 
	period = pmin + (c(1:p) - 1) * n.interval 
	cohort = cmin + (c(1:(a+p-1)) - 1) * n.interval 
    }
#}

if (apcmodel == "A")	x = x[,1:a]
if (apcmodel == "P")	x = x[,c(1,a+1:(p-1))]
if (apcmodel == "C")	x = x[,-2:-(a+p-1)]
if (apcmodel == "AP")	x = x[,1:(a+p-1)]
if (apcmodel == "AC")	x = x[,c(-(a+1):-(a+p-1))]
if (apcmodel == "PC")	x = x[,-2:-a]
if (apcmodel == "APC") {
	
	#########################################################################
	######## Intrinsic  estimator 
	#########################################################################
	if (apc == 0)	{
	   x1 = x[,1:a] 
	   x2 = x[,-1:-a]
	 
	  xx = x
		v <- eigen(t(xx) %*% xx)$vector
		x <- xx %*% v[,  - (ncol(v))]
	 }

	#########################################################################
	######## constrained estimator 
	#########################################################################
	if ((apc > 0) && (apc < 4))	{
	 	x = constr(nr=a, nc=p, c1=c1, c2=c2, cc=cc, apc = apc, lindex=lindex, lvalue=lvalue)  ### [,-1]
		if ( eigen(t(x)%*%x)$val[ncol(x)] < 1e-7 )
			stop (paste("constraint is not specified correctly, please make changes"))
	 }
 }	

##############################################################
###### get exposure info for glm model (loglinear, ... )
##############################################################

if ( sum(is.na(n.risk)) >0 )
	n.risk = rep(1, a*p)

else  { 
	if ( 1*(header == T) + 1*(c(dim(n.risk)[1]) > c(dim(r)[1]) ) +1*(c(dim(n.risk)[2]) > c(dim(r)[2]))==3 )
		n.risk = n.risk [-1, -1]
	n.risk = c(t(n.risk))
 }

######################################################################################
######    Calculate the number of events based on the rate and exposure  #############
######################################################################################

rr = c(t(r))*c(n.risk)

############################################################################################################
############################### scale the rate by the unit (e.g. 10^-5) for loglinear regression ###########
############################################################################################################

if ( (sum(abs( c(n.risk) - rep(1, a*p))) > 0) && (fam != "logit") )
	rr = rr* Scale

rr = round(rr,0)


############################################################################################################
############################### Getting submatrix for Delta method for Varince Estimates of Period and cohort ###########
############################################################################################################

if (apcmodel == "AC" || apcmodel == "AP")	{
  x1 = x[,1:a] 
  x2 = x[,-1:-a]
 } 
if (apcmodel == "PC")	{
  #### need more work here Shouls just switch the A and P to use AP model for PC data
  stop(paste("PC model is not recommended, use AP or AC model, instead. Or simply run AC model on transposed data"))
 }

############################################################################################################
############################### Fit GLM model to data with poisson, logit or quisi-lik    ##################
############################################################################################################

if (fam == "loglin") {
  if (apcmodel == "APC" && apc == 0)
		fit = glm(rr~x-1+offset(log(n.risk)), family=poisson (link=log) )
  else 
    fit = glm(rr~x-1+offset(log(n.risk)), family=poisson (link=log) )
}

if (fam == "qlik") {
  if (apcmodel == "APC" && apc == 0)
    fit = glm(rr~x-1+offset(log(n.risk)), family=quasipoisson(link=log) )
  else 
    fit = glm(rr~x-1+offset(log(n.risk)), family=quasipoisson(link=log) )
}

if (fam == "logit") {
  if (apcmodel == "APC" && apc == 0)
    fit = glm(rr~x-1, family=binomial (link=logit) )
  else 
    fit = glm(rr~x-1, family=binomial (link=logit) )
}

if (apcmodel == "A") {
	b.var = summary(fit,corr=T)$corr[-1,-1]
	b.lastse = sqrt(c(matrix(1,1,a-1)%*%diag(summary(fit)$coef[-1,2])%*%b.var%*%diag(summary(fit)$coef[-1,2])%*%matrix(1,a-1,1)))
	a.last=c(-sum(fit$coef[-1]), b.lastse, tt<- -sum(fit$coef[-1])/b.lastse, 2*(1-pt(abs(tt),fit$df.resid)))
	b.se = rbind(summary(fit)$coef,a.last)
	if (Plot == T) {
		plot(age,b.se[-1,1],type='b',ylim=range(cbind(b.se[-1,1],b.se[-1,1]-1.96*b.se[-1,2],b.se[-1,1]+1.96*b.se[-1,2])),ylab="")
		if (stderrplot == T) {
			lines(age, b.se[-1,1]-1.96*b.se[-1,2],lty=2)
			lines(age, b.se[-1,1]+1.96*b.se[-1,2],lty=2)
			title(main="Age effect & 95% CI")
 		 }
		else 
			title(main="Age effect")
	 }
 }

if (apcmodel == "P") {
	b.var = summary(fit,corr=T)$corr[-1,-1]
	b.lastse = sqrt(c(matrix(1,1,p-1)%*%diag(summary(fit)$coef[-1,2])%*%b.var%*%diag(summary(fit)$coef[-1,2])%*%matrix(1,p-1,1)))
	p.last=c(-sum(fit$coef[-1]), b.lastse, tt<- -sum(fit$coef[-1])/b.lastse, 2*(1-pt(abs(tt),fit$df.resid)))
	b.se = rbind(summary(fit)$coef,p.last)
	if (Plot == T) {
		plot(period,b.se[-1,1],type='b',ylim=range(cbind(b.se[-1,1],b.se[-1,1]-1.96*b.se[-1,2],b.se[-1,1]+1.96*b.se[-1,2])),ylab="")
		if (stderrplot == T) {
			lines(period, b.se[-1,1]-1.96*b.se[-1,2],lty=2)
			lines(period, b.se[-1,1]+1.96*b.se[-1,2],lty=2)
			title(main="Period effect & 95% CI")
		 }
		else 
			title(main="Period effect")

	 }
 }

if (apcmodel == "C") {
	b.var = summary(fit,corr=T)$corr[-1,-1]
	b.lastse = sqrt(c(matrix(1,1,a+p-2)%*%diag(summary(fit)$coef[-1,2])%*%b.var%*%diag(summary(fit)$coef[-1,2])%*%matrix(1,a+p-2,1)))
	c.last=c(-sum(fit$coef[-1]), b.lastse, tt<- -sum(fit$coef[-1])/b.lastse, 2*(1-pt(abs(tt),fit$df.resid)))
	b.se = rbind(summary(fit)$coef,c.last)
	if (Plot == T) {
		plot(cohort,b.se[-1,1],type='b',ylim=range(cbind(b.se[-1,1],b.se[-1,1]-1.96*b.se[-1,2],b.se[-1,1]+1.96*b.se[-1,2])),ylab="")
		if (stderrplot == T) {
			lines(cohort, b.se[-1,1]-1.96*b.se[-1,2],lty=2)
			lines(cohort, b.se[-1,1]+1.96*b.se[-1,2],lty=2)
			title(main="Cohort effect & 95% CI")
		 }
		else 
			title(main="Cohort effect")

	 }
 }

if (apcmodel == "AP") {
	b.var = summary(fit,corr=T)$corr[2:a,2:a]
	b.lastse = sqrt(c(matrix(1,1,a-1)%*%diag(summary(fit)$coef[2:a,2])%*%b.var%*%diag(summary(fit)$coef[2:a,2])%*%matrix(1,a-1,1)))
	a.last=c(-sum(fit$coef[2:a]), b.lastse, tt<- -sum(fit$coef[2:a])/b.lastse, 2*(1-pt(abs(tt),fit$df.resid)))
	b.var = summary(fit,corr=T)$corr[a+1:(p-1),a+1:(p-1)]
	b.lastse = sqrt(c(matrix(1,1,p-1)%*%diag(summary(fit)$coef[-1:-a,2])%*%b.var%*%diag(summary(fit)$coef[-1:-a,2])%*%matrix(1,p-1,1)))
	p.last=c(-sum(fit$coef[a+1:(p-1)]), b.lastse, tt<- -sum(fit$coef[a+1:(p-1)])/b.lastse, 2*(1-pt(abs(tt),fit$df.resid)))
	b.se = rbind(summary(fit)$coef[1:a,],a.last,summary(fit)$coef[a+1:(p-1),], p.last)

	####### Delta Method for Var(P) ######
	diag.mu = diag(fit$fitted)
	b.var.delta =  solve(t(x2)%*%diag.mu%*%x2)%*%(t(x2)%*%diag.mu%*%x1)%*%diag(summary(fit)$coef[1:a,2])%*%summary(fit,corr=T)$corr[1:a,1:a]%*%diag(summary(fit)$coef[1:a,2])%*%t(x1)%*%diag.mu%*%x2%*%solve(t(x2)%*%diag.mu%*%x2)
	bse.delta = sqrt(diag(b.var.delta))
	b.last.se.delta = sqrt(c(matrix(1,1,p-1)%*%b.var.delta%*%matrix(1,p-1,1)))
	
	p.delta=cbind (est<- c(fit$coef[-1:-a], -sum(fit$coef[-1:-a])), bse.est<-c(bse.delta, b.last.se.delta), 
	                                 tt<- est/bse.est, 2*(1-pt(abs(tt),fit$df.resid)))
	
	b.se.delta = rbind(summary(fit)$coef[1:a,],a.last, p.delta)
	
	if (Plot == T) {
		par(mfrow=c(2,1),new=F, fig=c(0,1,0.48,1),cex=pcex)
		plot (age, b.se[1:a+1,1],type='b',ylim=range(cbind(b.se[1:a+1,1],b.se[1:a+1,1]-1.96*b.se[1:a+1,2],b.se[1:a+1,1]+1.96*b.se[1:a+1,2])),ylab="")
		if (stderrplot == T) {
			lines(age, b.se[1:a+1,1]-1.96*b.se[1:a+1,2],lty=2)
			lines(age, b.se[1:a+1,1]+1.96*b.se[1:a+1,2],lty=2)
			title(main="Age effect & 95% CI")
		 }
		else 
			title(main="Age effect")

		par(mfrow=c(2,1),new=T, fig=c(0,1,0,0.52),cex=pcex)
		plot(period,b.se[-1:-(a+1),1],type='b',ylim=range(cbind(b.se[-1:-(a+1),1],b.se[-1:-(a+1),1]-1.96*b.se[-1:-(a+1),2],b.se[-1:-(a+1),1]+1.96*b.se[-1:-(a+1),2])),ylab="")
		if (stderrplot == T) {
			lines(period, b.se[-1:-(a+1),1]-1.96*b.se[-1:-(a+1),2],lty=2)
			lines(period, b.se[-1:-(a+1),1]+1.96*b.se[-1:-(a+1),2],lty=2)
			title(main="Period effect & 95% CI")
		 }
		else 
			title(main="Period effect")

		par(mfrow=c(1,1))
	}
 }

if (apcmodel == "AC") {
	b.var = summary(fit,corr=T)$corr[2:a,2:a]
	b.lastse = sqrt(c(matrix(1,1,a-1)%*%diag(summary(fit)$coef[2:a,2])%*%b.var%*%diag(summary(fit)$coef[2:a,2])%*%matrix(1,a-1,1)))
	a.last=c(-sum(fit$coef[2:a]), b.lastse, tt<- -sum(fit$coef[2:a])/b.lastse, 2*(1-pt(abs(tt),fit$df.resid)))
	b.var = summary(fit,corr=T)$corr[-1:-a,-1:-a]
	b.lastse = sqrt(c(matrix(1,1,a+p-2)%*%diag(summary(fit)$coef[-1:-a,2])%*%b.var%*%diag(summary(fit)$coef[-1:-a,2])%*%matrix(1,a+p-2,1)))
	c.last=c(-sum(fit$coef[-1:-a]), b.lastse, tt<- -sum(fit$coef[-1:-a])/b.lastse, 2*(1-pt(abs(tt),fit$df.resid)))
	b.se = rbind(summary(fit)$coef[1:a,],a.last,summary(fit)$coef[-1:-a,], c.last)
	out.PCA = b.se
	
	####### Delta Method for Var(C) ######
	diag.mu = diag(fit$fitted)
	b.var.delta =  solve(t(x2)%*%diag.mu%*%x2)%*%(t(x2)%*%diag.mu%*%x1)%*%diag(summary(fit)$coef[1:a,2])%*%summary(fit,corr=T)$corr[1:a,1:a]%*%diag(summary(fit)$coef[1:a,2])%*%t(x1)%*%diag.mu%*%x2%*%solve(t(x2)%*%diag.mu%*%x2)
  bse.delta = sqrt(diag(b.var.delta))
	b.last.se.delta = sqrt(c(matrix(1,1,a+p-2)%*%b.var.delta%*%matrix(1,a+p-2,1)))
	c.delta=cbind (est<- c(fit$coef[a+1:(a+p-2)], -sum(fit$coef[a+1:(a+p-2)])), bse.est<-c(bse.delta, b.last.se.delta), tt<- est/bse.est, 2*(1-pt(abs(tt),fit$df.resid)))
	b.se.delta = rbind(summary(fit)$coef[1:a,],a.last, c.delta)
	par.data = b.se.delta

	if (Plot == T) {
		par(mfrow=c(2,1))
	  par(mfrow=c(2,1),new=F, fig=c(0,1,0.48,1),cex=pcex)
		plot (age, b.se[1:a+1,1],type='b',ylim=range(cbind(b.se[1:a+1,1],b.se[1:a+1,1]-1.96*b.se[1:a+1,2],b.se[1:a+1,1]+1.96*b.se[1:a+1,2])),ylab="")
		if (stderrplot == T) {
			lines(age, b.se[1:a+1,1]-1.96*b.se[1:a+1,2],lty=2)
			lines(age, b.se[1:a+1,1]+1.96*b.se[1:a+1,2],lty=2)
			title(main="Age effect & 95% CI")
		 }
		else 
			title(main="Age effect")
		
		par(mfrow=c(2,1))
		par(mfrow=c(2,1),new=T, fig=c(0,1,0,0.52),cex=pcex)
		plot(cohort,b.se[-1:-(a+1),1],type='b',ylim=range(cbind(b.se[-1:-(a+1),1],b.se[-1:-(a+1),1]-1.96*b.se[-1:-(a+1),2],b.se[-1:-(a+1),1]+1.96*b.se[-1:-(a+1),2])),ylab="")
		if (stderrplot == T) {
			lines(cohort, b.se[-1:-(a+1),1]-1.96*bse.est,lty=2)  ### ,col=2)
			lines(cohort, b.se[-1:-(a+1),1]+1.96*bse.est,lty=2)  ### ,col=2)
			title(main="Cohort effect & 95% CI")
		 }
		else 
			title(main="Cohort effect")

		par(mfrow=c(1,1))
	}
 }

if (apcmodel == "APC") {

	#########################################################################
	######## Intrinsic estimator method -- Principal Components Analysis 
	#########################################################################
 
 if (apc == 0) {
	b = as.vector(v %*% as.matrix(c(fit$coef, 0)))
	bvar.1 = diag(summary.glm(fit)$coef[, 2]) %*% 
				summary.glm(fit,corr=T)$corr %*% diag(summary.glm(fit)$coef[, 2])
	bvarall = v %*% cbind(rbind(bvar.1, 0), 0) %*% t(v)
	b.int = b[1]
	b.int.var = bvarall[1,1]
	b = b[-1]
	a.se = sqrt(matrix(-1, 1, a - 1) %*% bvarall[2:a, 2:a] %*% matrix(-1, a - 1, 1))
	p.se = sqrt(matrix(-1, 1, p - 1) %*% bvarall[1+a:(a + p - 2), 1+a:(a + p - 2)] %*% matrix(-1, p - 1, 1))
	c.se = sqrt(matrix(-1, 1, a+p-2) %*% bvarall[1+(a+p-1):(2*(a+p-2)), 1+(a+p-1):(2*(a+p-2))]%*% matrix(-1,a+p-2,1))
	b.se = rbind( c(b.int, sqrt(b.int.var), b.int.t <-b.int/sqrt(b.int.var), 2*(1-pt(abs(b.int.t), df=fit$df.resid))),
	              cbind(b[1:(a-1)], sqrt(diag(bvarall)[2:a]), 
	                    tt <- b[1:(a-1)] /sqrt(diag(bvarall)[2:a]), 2*(1-pt(abs(tt), df=fit$df.resid)) ),
	              c( -sum(b[1:(a-1)]), a.se, tt <- -sum(b[1:(a-1)])/a.se, 2*(1-pt(abs(tt), df=fit$df.resid)) ),
	              cbind(b[a:(a+p-2)], sqrt(diag(bvarall)[1+a:(a+p-2)]), tt <- b[a:(a+p-2)] /sqrt(diag(bvarall)[1+a:(a+p-2)]), 2*(1-pt(abs(tt), df=fit$df.resid)) ),
	              c( -sum(b[a:(a+p-2)]), p.se, tt <- -sum(b[a:(a+p-2)])/p.se, 2*(1-pt(abs(tt), df=fit$df.resid)) ),
	              cbind(b[-1:-(a+p-2)], sqrt(diag(bvarall)[-1:-(a+p-1)]), tt <- b[-1:-(a+p-2)] /sqrt(diag(bvarall)[-1:-(a+p-1)]), 2*(1-pt(abs(tt), df=fit$df.resid)) ),
	              c( -sum(b[-1:-(a+p-2)]), c.se, tt <- -sum(b[-1:-(a+p-2)])/c.se, 2*(1-pt(abs(tt), df=fit$df.resid)) )       
	              )
	out.PCA = b.se
	
	####### Delta Method for Var(C) ######
	diag.mu=diag(fit$fitted)
	b.var.delta =  solve(t(x2)%*%diag.mu%*%x2)%*%(t(x2)%*%diag.mu%*%x1)%*%bvarall[1:a,1:a]%*%t(x1)%*%diag.mu%*%x2%*%solve(t(x2)%*%diag.mu%*%x2)
	bse.delta = sqrt(diag(b.var.delta))
	b.last.se.delta.p = sqrt(c(matrix(1,1,p-1)%*%b.var.delta[1:(p-1),1:(p-1)]%*%matrix(1,p-1,1)))
	b.last.se.delta.c = sqrt(c(matrix(1,1,a+p-2)%*%b.var.delta[-1:-(p-1),-1:-(p-1)]%*%matrix(1,a+p-2,1)))
	p.delta=cbind (est<- c(b[a:(a+p-2)], -sum(b[a:(a+p-2)])), bse.est<-c(bse.delta[1:(p-1)], b.last.se.delta.p), tt<- est/bse.est, 2*(1-pt(abs(tt),fit$df.resid)))
	c.delta=cbind (est<- c(b[-1:-(a+p-2)], -sum(b[-1:-(a+p-2)])), bse.est<-c(bse.delta[-1:-(p-1)], b.last.se.delta.c), tt<- est/bse.est, 2*(1-pt(abs(tt),fit$df.resid)))
	
	b.se.delta = rbind(c(b.int, sqrt(b.int.var), b.int.t <-b.int/sqrt(b.int.var), 2*(1-pt(abs(b.int.t), df=fit$df.resid))),
       cbind(b[1:(a-1)], sqrt(diag(bvarall)[2:a]), 
       tt <- b[1:(a-1)] /sqrt(diag(bvarall)[2:a]), 2*(1-pt(abs(tt), df=fit$df.resid)) ),
       c( -sum(b[1:(a-1)]), a.se, tt <- -sum(b[1:(a-1)])/a.se, 2*(1-pt(abs(tt), df=fit$df.resid)) ),
       p.delta, c.delta)
     }			
   
	#########################################################################
	######## constrained estimator 
	#########################################################################
    if (apc > 0)  {
	if ( sum(abs(lindex))>0 && sum(abs(lvalue))>0 ) {	### vector constraint
		  	lorder = order(lindex)
			lindex = lindex[lorder]
			lvalue = lvalue[lorder]
			nindex= length(lindex)
			indexlast = lindex[nindex]
			valuelast = lvalue[nindex]
		
			if(apc == 1) {
				aa <- fit$coef[2:(a - 1)]
				aa <- c(aa[1:(indexlast - 1)], sum(-lvalue[-nindex] * aa[ lindex[-nindex] ])/valuelast,
					 aa[-1: - (indexlast - 1)])
				pp <- fit$coef[a:(a + p - 2)]
				coh <- fit$coef[-1:-(a + p - 2)]
			 }

			if(apc == 2) {
				aa <- fit$coef[2:a]
				pp <- fit$coef[(a + 1):(a + p - 2)]
				pp <- c(pp[1:(indexlast - 1)], sum(-lvalue[-nindex] * pp[ lindex[-nindex] ])/valuelast , 
					pp[-1: - (indexlast - 1)])
				coh <- fit$coef[-1:-(a + p - 2)]

			 }

			if(apc == 3) {
				aa <- fit$coef[2:a]
				pp <- fit$coef[(a + 1):(a + p - 1)]
				coh <- fit$coef[-1:-(a + p-1)]
				coh <- c(coh[1:(indexlast - 1)], sum(-lvalue[-nindex] * coh[ lindex[-nindex] ])/valuelast ,
					 coh[-1: - (indexlast - 1)])
			 }
	  }
	else {  		## non-vector constraint
		  if(apc == 1) {
			aa <- fit$coef[2:(a - 1)]
			aa <- c(aa[1:(c2 - 1)], cc * aa[c1], aa[-1: - (c2 - 1)])
			pp <- fit$coef[a:(a + p - 2)]
			coh <- fit$coef[(a + p - 1):length(fit$coef)]
		   }
		  if(apc == 2) {
			aa <- fit$coef[2:a]
			pp <- fit$coef[(a + 1):(a + p - 2)]
			pp <- c(pp[1:(c2 - 1)], cc * pp[c1], pp[-1: - (c2 - 1)])
			coh <- fit$coef[(a + p - 1):length(fit$coef)]
		   }
		  if(apc == 3) {
			aa <- fit$coef[2:a]
			pp <- fit$coef[(a + 1):(a + p - 1)]
			coh <- fit$coef[-1:-(a+p-1)]					### [(a + p):length(fit$coef)]
			coh <- c(coh[1:(c2 - 1)], cc * coh[c1], coh[-1: - (c2 - 1)])
		   }
	 }
	b.se = as.matrix(c(fit$coef[1], aa, -sum(aa), pp, -sum(pp), coh, -sum(coh)))
  }	

#####################################################################################
###### Plot the effects for APC model (intrinsic or constraint estiamtor) ##########
#####################################################################################

	if (Plot == T) {
		par(mfrow=c(2,2),cex=pcex)
		if ( abs(pylim[1]-pylim[2]) < 1e-12 ) {
		    if ( apcmodel != "APC" || apc == 0 )
			plot (age, b.se[1:a+1,1],type='l',ylim=range(cbind(b.se[1:a+1,1],b.se[1:a+1,1]-1.96*b.se[1:a+1,2],b.se[1:a+1,1]+1.96*b.se[1:a+1,2]),na.rm=T),ylab="")
		    else 
			plot (age, b.se[1:a+1,1],type='l',ylim=range(b.se[1:a+1,1],na.rm=T),ylab="")
		 }
		else 
			plot (age, b.se[1:a+1,1],type='l',ylim=pylim,ylab="")

		points(age, b.se[1:a+1,1])

		if (stderrplot == T && apc == 0) {
			lines(age, b.se[1:a+1,1]-1.96*b.se[1:a+1,2],lty=2)
			lines(age, b.se[1:a+1,1]+1.96*b.se[1:a+1,2],lty=2)
			title(main="Age effect & 95% CI")
		 }
		else 
			title(main="Age effect")

		if ( abs(pylim[1]-pylim[2]) < 1e-12 ) {
		   if ( apcmodel != "APC" || apc == 0 )
			plot (period, b.se[(a+2):(a+p+1),1],type='l',ylim=range(cbind(b.se[(a+2):(a+p+1),1],b.se[(a+2):(a+p+1),1]-1.96*b.se[(a+2):(a+p+1),2],
				b.se[(a+2):(a+p+1),1]+1.96*b.se[(a+2):(a+p+1),2]),na.rm=T),ylab="")
		   else 
			plot (period, b.se[(a+2):(a+p+1),1],type='l',ylim=range(b.se[(a+2):(a+p+1),1],na.rm=T),ylab="")
  		 }
		else 
			plot (period, b.se[(a+2):(a+p+1),1],type='l',ylim=pylim,ylab="")

		points(period, b.se[(a+2):(a+p+1),1])
		if (stderrplot == T && apc == 0) {
			title(main="Period effect & 95% CI")
			lines(period, b.se[(a+2):(a+p+1),1]-1.96*p.delta[,2],lty=2 , col=1)
			lines(period, b.se[(a+2):(a+p+1),1]+1.96*p.delta[,2],lty=2 , col=1)
		 }
		else 
			title(main="Period effect")

		par(mfrow=c(2,1),new=T, fig=c(0,1,0,0.52),cex=pcex)
		if ( abs(pylim[1]-pylim[2]) < 1e-12 ) {
		    if ( apcmodel != "APC" || apc == 0 )
			plot (cohort,b.se[-1:-(a+p+1),1],type='l',ylim=range(cbind(b.se[-1:-(a+p+1),1],b.se[-1:-(a+p+1),1]-1.96*b.se[-1:-(a+p+1),2],b.se[-1:-(a+p+1),1]+1.96*b.se[-1:-(a+p+1),2]),na.rm=T),ylab="")
		    else 
			plot (cohort,b.se[-1:-(a+p+1),1],type='l',ylim=range(b.se[-1:-(a+p+1),1],na.rm=T),ylab="")
		 }
		else 
			plot (cohort,b.se[-1:-(a+p+1),1],type='l',ylim=pylim,ylab="")

		points(cohort, b.se[-1:-(a+p+1),1])
		if (stderrplot == T && apc == 0) {
			title(main="Cohort effect & 95% CI")
			lines(cohort, b.se[-1:-(a+p+1),1]-1.96*c.delta[,2],lty=2, col=1)
			lines(cohort, b.se[-1:-(a+p+1),1]+1.96*c.delta[,2],lty=2, col=1)
		}
		else 
			title(main="Cohort effect")
		par(mfrow=c(1,1))
	}
    }		######## Finishing either constrained estimator or intrinsic estimator.

################################################################
######## relabel the variables in output

b.label = dimnames(b.se)[[1]]
b.label[1] = "Intercept"
if (apcmodel == "A") 
	for (i in 1:a) 	b.label[i+1] = paste("Age",as.character(age[i]))

if (apcmodel == "P") 
	for (i in 1:p) 	b.label[i+1] = paste("Period",as.character(period[i]))

if (apcmodel == "C") 
	for (i in 1:(a+p-1)) 	b.label[i+1] = paste("Cohort",as.character(cohort[i]))

if (apcmodel == "AP") {
	for (i in 1:a) 	b.label[i+1] = paste("Age",as.character(age[i]))
	for (i in (a+1):(a+p)) b.label[i+1] = paste("Period",as.character(period[i-a]))
 }

if (apcmodel == "AC") {
	for (i in 1:a) 	b.label[i+1] = paste("Age",as.character(age[i]))
	for (i in (a+1):(2*a+p-1)) b.label[i+1] = paste("Cohort",as.character(cohort[i-a]))
 }

if (apcmodel == "APC") {
	for (i in 1:a) 	b.label[i+1] = paste("Age",as.character(age[i]))
	for (i in (a+1):(a+p)) b.label[i+1] = paste("Period",as.character(period[i-a]))
	for (i in (a+p+1):(2*a+2*p-1)) b.label[i+1] = paste("Cohort",as.character(cohort[i-a-p]))
 }
dimnames(b.se)[[1]] = b.label

if (apcmodel == "AC")
  dimnames(b.se.delta)[[1]] = b.label

out = vector("list")
out$model = summary(fit)$call
if (apcmodel == "APC" && apc > 0) {
	if (apc == 1) 
		out$constraint = "Age"
	if (apc == 2)
		out$constraint = "Period"
	if (apc == 3)
		out$constraint = "Cohort"
	if (sum(lindex) > 0)
		out$constraint.para = rbind(lindex, lvalue)
	if (sum(lindex) == 0) {
		lindex = c(c1, c2) ; lvalue = c(-cc, 1)
		out$constraint.para = rbind(lindex, lvalue) 
	 }
}
dev = matrix(c(summary(fit)$deviance, summary(fit)$df.resid),1,2)
dimnames(dev)[[2]]=c("Deviance", "DF")
out$deviance = dev
out$pearson.chisq = sum((resid(fit,type="pearson"))^2)
p.val = 1 - pchisq(dev[1],dev[2])
names(p.val) = "p.value"
out$p.val = p.val

if (fam=="qlik")
  out$dispersion = summary(fit)$dispersion

if (apc < 1) 
  dimnames(b.se)[[2]] = dimnames(summary(fit)$coef)[[2]]

out$parameter = b.se

###### This part should not be conducted as the AP are perpendicular, making var(P) extremely small ######
##if (apcmodel == "AP") {
##  par.delta = b.se.delta
##  dimnames(par.delta)[[1]]=dimnames(b.se)[[1]]
##  dimnames(par.delta)[[2]]=dimnames(summary(fit)$coef)[[2]]
##  out$parameter.delta = par.delta
##}

if (apcmodel == "AC") {
  par.delta = b.se.delta
  dimnames(par.delta)[[1]]=dimnames(b.se)[[1]]
  dimnames(par.delta)[[2]]=dimnames(summary(fit)$coef)[[2]]
  out$parameter = par.delta
#  out$parameter.PCA = out.PCA
}
if (apcmodel == "APC" ) {
  if (apc == 0) {
  par.delta = b.se.delta
  dimnames(par.delta)[[1]]=dimnames(b.se)[[1]]
  dimnames(par.delta)[[2]]=dimnames(summary(fit)$coef)[[2]]
  out$parameter = par.delta
#  out$parameter.PCA = out.PCA
  }
  if (apc > 0) {
    
    
  }
}
return(out)
}
