apclinfit_run <-
function(r, apcmodel="AP", header = F, Plot = F,  pylim = c(0, 0), transform="log", 
  ref = c(0,0,0), c1=1, c2=1, cc=1, apc=0, lindex=0, lvalue=0, amin = 1, pmin = 1, cmin = 1, gapyear = 1, 
  stderrplot=T, ModelDiag=F, pcex=1) {
##################################################################################
## fit APC model to a matrix of rates using linear model least-squares method.
## No full models, only one factor or two factor models. 
################# Estimation of Stderr ##############
##################################################################################

a = nrow(r)
p = ncol(r)
x = apcmat(a, p, ref=ref)

if (header == T) {
  if (length(dimnames(r)[[1]]) > 0)
    age = as.numeric(dimnames(r)[[1]])
  else
    age = amin + (c(1:a) - 1) * gapyear 
  
  if (length(dimnames(r)[[2]]) > 0)
    period = as.numeric(dimnames(r)[[2]])
  else 
    period = pmin + (c(1:p) - 1) * gapyear 
  if (1*(age[1] > 1) + 1*(period[1] > 1) > 1)
    cohort = seq(min(period)-max(age), max(period)-min(age),age[3]-age[2])
  else 
    cohort = cmin + (c(1:(a+p-1)) - 1) * gapyear 
}
else {
	  age    = amin + (c(1:a) - 1) * gapyear 
	  period = pmin + (c(1:p) - 1) * gapyear 
	  cohort = cmin + (c(1:(a+p-1)) - 1) * gapyear 
}

if (apcmodel == "A")	x = x[,1:a]
if (apcmodel == "P")	x = x[,c(1,a+1:(p-1))]
if (apcmodel == "C")	x = x[,-2:-(a+p-1)]
if (apcmodel == "AP")	x = x[,1:(a+p-1)]
if (apcmodel == "AC")	x = x[,c(-(a+1):-(a+p-1))]
if (apcmodel == "PC")	x = x[,-2:-a]
if (apcmodel == "APC") {
	
	#########################################################################
	###################   Intrinsic  estimator   ############################
  
	#########################################################################
	if (apc == 0)	{
	  x1 = x[,1:a] 
	  x2 = x[,-1:-a]
	  
		xx = x  
		v <- eigen(t(xx) %*% xx)$vector
		x <- xx %*% v[,  - (ncol(v))]
	 }

	#########################################################################
	######## constrained estimator 
	#########################################################################
	if ((apc > 0) && (apc < 4))	{
	 x = constr(nr=a, nc=p, c1=c1, c2=c2, cc=cc, apc = apc, lindex=lindex, lvalue=lvalue) 
	 }
 }	

rr = c(t(r))


############################################################################################################
############################### Fitting Linear Model to APC data  ##########################################
############################################################################################################


if (substr(transform,1,2) == "id" || substr(transform,1,2) == "ID") 
	fit = lm(rr~x-1)
if (substr(transform,1,3) == "log" || substr(transform,1,3) == "Log")
	fit = lm(log(rr)~x-1)

if (apcmodel == "A") {
	b.var = summary(fit,corr=T)$corr[-1,-1]
	b.lastse = sqrt(c(matrix(1,1,a-1)%*%diag(summary(fit)$coef[-1,2])%*%b.var%*%diag(summary(fit)$coef[-1,2])%*%matrix(1,a-1,1)))
	a.last=c(-sum(fit$coef[-1]), b.lastse, tt<- -sum(fit$coef[-1])/b.lastse, 2*(1-pt(abs(tt),fit$df)))
	b.se = rbind(summary(fit)$coef,a.last)
	if (Plot == T) {
		plot(age,b.se[-1,1],type='b',ylim=range(cbind(b.se[-1,1],b.se[-1,1]-1.96*b.se[-1,2],b.se[-1,1]+1.96*b.se[-1,2])),ylab="")
	  points(age, b.se[-1,1])
		if (stderrplot == T) {
			lines(age, b.se[-1,1]-1.96*b.se[-1,2],lty=2)
			lines(age, b.se[-1,1]+1.96*b.se[-1,2],lty=2)
			title(main="Age effect & 95% CI")
 		 }
		else 
			title(main="Age effect")
	}
 }

if (apcmodel == "P") {
	b.var = summary(fit,corr=T)$corr[-1,-1]
	b.lastse = sqrt(c(matrix(1,1,p-1)%*%diag(summary(fit)$coef[-1,2])%*%b.var%*%diag(summary(fit)$coef[-1,2])%*%matrix(1,p-1,1)))
	p.last=c(-sum(fit$coef[-1]), b.lastse, tt<- -sum(fit$coef[-1])/b.lastse, 2*(1-pt(abs(tt),fit$df)))
	b.se = rbind(summary(fit)$coef,p.last)
	if (Plot == T) {
		plot(period,b.se[-1,1],type='b',ylim=range(cbind(b.se[-1,1],b.se[-1,1]-1.96*b.se[-1,2],b.se[-1,1]+1.96*b.se[-1,2])),ylab="")
	  points(period, b.se[-1,1])
		if (stderrplot == T) {
			lines(period, b.se[-1,1]-1.96*b.se[-1,2],lty=2)
			lines(period, b.se[-1,1]+1.96*b.se[-1,2],lty=2)
			title(main="Period effect & 95% CI")
		 }
		else 
			title(main="Period effect")

	 }
 }

if (apcmodel == "C") {
	b.var = summary(fit,corr=T)$corr[-1,-1]
	b.lastse = sqrt(c(matrix(1,1,a+p-2)%*%diag(summary(fit)$coef[-1,2])%*%b.var%*%diag(summary(fit)$coef[-1,2])%*%matrix(1,a+p-2,1)))
	c.last=c(-sum(fit$coef[-1]), b.lastse, tt<- -sum(fit$coef[-1])/b.lastse, 2*(1-pt(abs(tt),fit$df)))
	b.se = rbind(summary(fit)$coef,c.last)
	if (Plot == T) {
		plot(cohort,b.se[-1,1],type='b',ylim=range(cbind(b.se[-1,1],b.se[-1,1]-1.96*b.se[-1,2],b.se[-1,1]+1.96*b.se[-1,2])),ylab="")
	  points(cohort, b.se[-1,1])
		if (stderrplot == T) {
			lines(cohort, b.se[-1,1]-1.96*b.se[-1,2],lty=2)
			lines(cohort, b.se[-1,1]+1.96*b.se[-1,2],lty=2)
			title(main="Cohort effect & 95% CI")
		 }
		else 
			title(main="Cohort effect")

	 }
 }

if (apcmodel == "AP") {
	b.var = summary(fit,corr=T)$corr[2:a,2:a]
	b.lastse = sqrt(c(matrix(1,1,a-1)%*%diag(summary(fit)$coef[2:a,2])%*%b.var%*%diag(summary(fit)$coef[2:a,2])%*%matrix(1,a-1,1)))
	a.last=c(-sum(fit$coef[2:a]), b.lastse, tt<- -sum(fit$coef[2:a])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

	b.var = summary(fit,corr=T)$corr[a+1:(p-1),a+1:(p-1)]
	b.lastse = sqrt(c(matrix(1,1,p-1)%*%diag(summary(fit)$coef[-1:-a,2])%*%b.var%*%diag(summary(fit)$coef[-1:-a,2])%*%matrix(1,p-1,1)))
	p.last=c(-sum(fit$coef[a+1:(p-1)]), b.lastse, tt<- -sum(fit$coef[a+1:(p-1)])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

	b.se = rbind(summary(fit)$coef[1:a,],a.last,summary(fit)$coef[a+1:(p-1),], p.last)

	### calculate SS and MSE for age (row) and period (col) effects
	rlog = log(r)
	r.amean = apply(rlog, 1, mean)
	ss.a = sum((rlog-outer(r.amean, rep(1,p)))^2)
	mse.a = ss.a/(a-1)
	f.a = mse.a/summary(fit)$sig
	df.2 = summary(fit)$df[2]
	pval.a = 2*(1 - pf (f.a, a-1, df.2) )

	r.pmean=apply(rlog,2,mean) 
	ss.p = sum((rlog-outer(rep(1,a), r.pmean))^2)
	mse.p = ss.p/(p-1)
	f.p = mse.p/summary(fit)$sig
	pval.p = 2*(1 - pf (f.p, p-1, df.2) )

	p.val = 2*(1-pf(summary(fit)$f[1],summary(fit)$f[2],summary(fit)$f[3]))

	### list F-test statistic for total, age, and period effects
	F.ap =  rbind(c(summary(fit)$f, p.val), c(f.a, a-1, df.2, pval.a), c(f.p, p-1, df.2, pval.p))
	dimnames(F.ap)[[2]] = c("F.Stat", "numdf", "dendf", "p.value")
	dimnames(F.ap)[[1]] = c("Model","Age","Period")

	### SS and MSE list
	df.mod = summary(fit)$f[2]
#	mse.mod = summary(fit)$f[1]*summary(fit)$sig
#	ss.mod = mse.mod*df.mod
	ss.mod = sum((rlog - mean(rlog))^2)
	mse.mod = ss.mod / df.mod

	SS.ap = rbind(c(ss.mod, mse.mod, df.mod), c(ss.a, mse.a, a-1), 
		c(ss.p, mse.p, p-1) )
	dimnames(SS.ap)[[1]] = c("Model","Age","Period")
	dimnames(SS.ap)[[2]] = c("SS", "MSE", "df")


	if (Plot == T) {
		par(mfrow=c(2,1), cex=pcex)
		plot (age, b.se[1:a+1,1],type='b',ylim=range(cbind(b.se[1:a+1,1],b.se[1:a+1,1]-1.96*b.se[1:a+1,2],b.se[1:a+1,1]+1.96*b.se[1:a+1,2])),ylab="")
		points(age, b.se[1:a+1,1])
		if (stderrplot == T) {
			lines(age, b.se[1:a+1,1]-1.96*b.se[1:a+1,2],lty=2)
			lines(age, b.se[1:a+1,1]+1.96*b.se[1:a+1,2],lty=2)
			title(main="Age effect & 95% CI")
		 }
		else 
			title(main="Age effect")


		plot(period,b.se[-1:-(a+1),1],type='b',ylim=range(cbind(b.se[-1:-(a+1),1],b.se[-1:-(a+1),1]-1.96*b.se[-1:-(a+1),2],b.se[-1:-(a+1),1]+1.96*b.se[-1:-(a+1),2])),ylab="")
		points(period, b.se[-1:-(a+1),1])
		if (stderrplot == T) {
			lines(period, b.se[-1:-(a+1),1]-1.96*b.se[-1:-(a+1),2],lty=2)
			lines(period, b.se[-1:-(a+1),1]+1.96*b.se[-1:-(a+1),2],lty=2)
			title(main="Period effect & 95% CI")
		 }
		else 
			title(main="Period effect")

		par(mfrow=c(1,1))
	}
 }

if (apcmodel == "AC") {
	b.var = summary(fit,corr=T)$corr[2:a,2:a]
	b.lastse = sqrt(c(matrix(1,1,a-1)%*%diag(summary(fit)$coef[2:a,2])%*%b.var%*%diag(summary(fit)$coef[2:a,2])%*%matrix(1,a-1,1)))
	a.last=c(-sum(fit$coef[2:a]), b.lastse, tt<- -sum(fit$coef[2:a])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

	b.var = summary(fit,corr=T)$corr[-1:-a,-1:-a]
	b.lastse = sqrt(c(matrix(1,1,a+p-2)%*%diag(summary(fit)$coef[-1:-a,2])%*%b.var%*%diag(summary(fit)$coef[-1:-a,2])%*%matrix(1,a+p-2,1)))
	c.last=c(-sum(fit$coef[-1:-a]), b.lastse, tt<- -sum(fit$coef[-1:-a])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

	b.se = rbind(summary(fit)$coef[1:a,],a.last,summary(fit)$coef[-1:-a,], c.last)
	

	if (Plot == T) {
		par(mfrow=c(2,1),cex=pcex)
		plot (age, b.se[1:a+1,1],type='b',ylim=range(cbind(b.se[1:a+1,1],b.se[1:a+1,1]-1.96*b.se[1:a+1,2],b.se[1:a+1,1]+1.96*b.se[1:a+1,2])),ylab="")
		points(age, b.se[1:a+1,1])
		if (stderrplot == T) {
			lines(age, b.se[1:a+1,1]-1.96*b.se[1:a+1,2],lty=2)
			lines(age, b.se[1:a+1,1]+1.96*b.se[1:a+1,2],lty=2)
			title(main="Age effect & 95% CI")
		 }
		else 
			title(main="Age effect")

		plot(cohort,b.se[-1:-(a+1),1],type='b',ylim=range(cbind(b.se[-1:-(a+1),1],b.se[-1:-(a+1),1]-1.96*b.se[-1:-(a+1),2],b.se[-1:-(a+1),1]+1.96*b.se[-1:-(a+1),2])),ylab="")
		points(cohort, b.se[-1:-(a+1),1])
		if (stderrplot == T) {
			lines(cohort, b.se[-1:-(a+1),1]-1.96*b.se[-1:-(a+1),2],lty=2)
			lines(cohort, b.se[-1:-(a+1),1]+1.96*b.se[-1:-(a+1),2],lty=2)
			
			title(main="Cohort effect & 95% CI")
		 }
		else 
			title(main="Cohort effect")

		par(mfrow=c(1,1))
	}
 }

if (apcmodel == "APC") {
	#########################################################################
	######## Intrinsic estimator method -- Principal Components Analysis 
	#########################################################################
   if (apc == 0) {
	b.int = fit$coef[1]
	b = as.vector(v %*% as.matrix(c(fit$coef, 0)))
	bvar.1 = diag(summary.lm(fit)$coef[, 2]) %*% 
	  summary.lm(fit,corr=T)$corr %*% diag(summary.lm(fit)$coef[, 2])
	
	bvarall = v %*% cbind(rbind(bvar.1, 0), 0) %*% t(v)
	a.se = sqrt(matrix(-1, 1, a - 1) %*% bvarall[1+1:(a - 1), 1+1:(a - 1)] %*% matrix(-1, a - 1, 1))
	p.se = sqrt(matrix(-1, 1, p - 1) %*% bvarall[1+a:(a + p - 2), 1+a:(a + p - 2)] %*% matrix(-1, p - 1, 1))
	c.se = sqrt(matrix(-1, 1, a+p-2) %*% bvarall[1+(a+p-1):(2*(a+p-2)), 1+(a+p-1):(2*(a+p-2))]%*% matrix(-1,a+p-2,1))

	b.se = rbind( ######  summary.lm(fit)$coef[1,], ### Forgot to convert by the rotation matrix, now correct it ###
	      cbind(b[1:a], bs <- sqrt(diag(bvarall[1:a,1:a])), tt <- b[1:a] /bs, 2*(1-pt(abs(tt), df=fit$df.resid))),
			  c( -sum(b[1+1:(a-1)]), a.se, tt <- -sum(b[1+1:(a-1)])/a.se, 2*(1-pt(abs(tt), df=fit$df.resid) )),
			  cbind(b[1+a:(a+p-2)], sqrt(diag(bvarall)[1+a:(a+p-2)]), tt <- b[1+a:(a+p-2)] /sqrt(diag(bvarall)[1+a:(a+p-2)]), 2*(1-pt(abs(tt), df=fit$df.resid)) ),
			  c( -sum(b[1+a:(a+p-2)]), p.se, tt <- -sum(b[1+a:(a+p-2)])/p.se, 2*(1-pt(abs(tt), df=fit$df.resid)) ),
			  cbind(b[-1:-(a+p-1)], sqrt(diag(bvarall)[-1:-(a+p-1)]), tt <- b[-1:-(a+p-1)] /sqrt(diag(bvarall)[-1:-(a+p-1)]), 2*(1-pt(abs(tt), df=fit$df.resid)) ),
			  c( -sum(b[-1:-(a+p-1)]), c.se, tt <- -sum(b[-1:-(a+p-1)])/c.se, 2*(1-pt(abs(tt), df=fit$df.resid) )  ))
     }			
   
	#########################################################################
	######## constrained estimator 
	#########################################################################
    else {
	if ( sum(abs(lindex))>0 && sum(abs(lvalue))>0 ) {	### vector constraint
		  	lorder = order(lindex)
			lindex = lindex[lorder]
			lvalue = lvalue[lorder]
			nindex= length(lindex)
			indexlast = lindex[nindex]
			valuelast = lvalue[nindex]
	  }
	if ( sum(abs(lindex))>0 && sum(abs(lvalue))>0 ) {
			if(apc == 1) {
				aa <- fit$coef[2:(a - 1)]
				aa <- c(aa[1:(indexlast - 1)], sum(-lvalue[-nindex] * aa[ lindex[-nindex] ])/valuelast,
					 aa[-1: - (indexlast - 1)])
				pp <- fit$coef[a:(a + p - 2)]
				coh <- fit$coef[-1:-(a + p - 2)]
			 }

			if(apc == 2) {
				aa <- fit$coef[2:a]
				pp <- fit$coef[(a + 1):(a + p - 2)]
				pp <- c(pp[1:(indexlast - 1)], sum(-lvalue[-nindex] * pp[ lindex[-nindex] ])/valuelast , 
					pp[-1: - (indexlast - 1)])
				coh <- fit$coef[-1:-(a + p - 2)]

			 }

			if(apc == 3) {
				aa <- fit$coef[2:a]
				pp <- fit$coef[(a + 1):(a + p - 1)]
				coh <- fit$coef[-1:-(a + p-1)]
				coh <- c(coh[1:(indexlast - 1)], sum(-lvalue[-nindex] * coh[ lindex[-nindex] ])/valuelast ,
					 coh[-1: - (indexlast - 1)])
			 }
	  }
	else {  		## non-vector constraint
		  if(apc == 1) {
			aa <- fit$coef[2:(a - 1)]
			aa <- c(aa[1:(c2 - 1)], cc * aa[c1], aa[-1: - (c2 - 1)])
			pp <- fit$coef[a:(a + p - 2)]
			coh <- fit$coef[(a + p - 1):length(fit$coef)]
		   }
		  if(apc == 2) {
			aa <- fit$coef[2:a]
			pp <- fit$coef[(a + 1):(a + p - 2)]
			pp <- c(pp[1:(c2 - 1)], cc * pp[c1], pp[-1: - (c2 - 1)])
			coh <- fit$coef[(a + p - 1):length(fit$coef)]
		   }
		  if(apc == 3) {
			aa <- fit$coef[2:a]
			pp <- fit$coef[(a + 1):(a + p - 1)]
			coh <- fit$coef[(a + p):length(fit$coef)]
			coh <- c(coh[1:(c2 - 1)], cc * coh[c1], coh[-1: - (c2 - 1)])
		   }
	 }

#####################################
### Compute the standard errors
#####################################
	
	if (apc == 3) {
		b.var = summary(fit,corr=T)$corr[2:a,2:a]
		b.lastse = sqrt(c(matrix(1,1,a-1)%*%diag(summary(fit)$coef[2:a,2])%*%b.var%*%diag(summary(fit)$coef[2:a,2])%*%matrix(1,a-1,1)))
		a.last=c(-sum(fit$coef[2:a]), b.lastse, tt <- -sum(fit$coef[2:a])/b.lastse, 2*(1-pt(abs(tt),fit$df)) )

		b.var = summary(fit,corr=T)$corr[(a+1):(a+p-1),(a+1):(a+p-1)]
		b.lastse = sqrt(c(matrix(1,1,p-1)%*%diag(summary(fit)$coef[(a+1):(a+p-1),2])%*%b.var%*%diag(summary(fit)$coef[(a+1):(a+p-1),2])%*%matrix(1,p-1,1)))
		p.last=c(-sum(fit$coef[(a+1):(a+p-1)]), b.lastse, tt <- -sum(fit$coef[(a+1):(a+p-1)])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

		est <- fit$coef[-1:-(a+p-1)]
		b.var = summary(fit,corr=T)$corr[-1:-(a+p-1),-1:-(a+p-1)]

	####  compute the value for the last constrained estimates
		if ( sum(abs(lindex))>0 && sum(abs(lvalue))>0 ) {

			est.const.last = sum(-lvalue[-nindex] * est[ lindex[-nindex] ])/valuelast   ######## estimate of last constraint
			est = c(est[1:(indexlast - 1)], est.const.last ,  est[-1: - (indexlast - 1)])
			est.last = -sum(est)

			####  compute the variance for the last constrained estimates
	
			b.se.const = summary(fit)$coef[-1:-(a+p-1),2][lindex[-nindex]] ######## for the se of the param in the constraint ## sum(-lvalue[-nindex]*coh[lindex[-nindex]])/valuelast
			
			if (nindex ==2)
				b.var.const = lvalue[-nindex]*b.se.const*b.var[lindex[-nindex],lindex[-nindex]]*b.se.const*
				    lvalue[-nindex]/valuelast^2
			else 
				b.var.const = t(as.matrix(lvalue[-nindex]))%*%diag(b.se.const)%*%b.var[lindex[-nindex],lindex[-nindex]]%*%diag(b.se.const)%*%
				    as.matrix(lvalue[-nindex])/valuelast^2
			
			index = rep(0, a+p-3)
			index [lindex[-nindex]] = lvalue[-nindex]
			index.mat = rbind(diag(rep(1,a+p-3)), index)

			b.lastse = sqrt( sum( index.mat%*% diag(summary(fit)$coef[-1:-(a+p-1),2]) %*%b.var%*%diag(summary(fit)$coef[-1:-(a+p-1),2]) %*%t(index.mat) ))
			c.last=c(est.last, b.lastse, tt <- est.last/b.lastse, 2*(1-pt(abs(tt),fit$df)) )

			#### put all parameter estimates and se together 
				
			b.const.remain = c(est.const.last, sqrt(b.var.const), tt<-est.const.last/sqrt(b.var.const), 2*(1-pt(abs(tt),fit$df)) )
			if (nindex < (a+p-2) )
				b.const.remain = rbind(b.const.remain, summary(fit)$coef[-1:-(a+p-1+indexlast-1),])
			
			b.se = rbind(summary(fit)$coef[1:a,], a.last, summary(fit)$coef[(a+1):(a+p-1),], p.last,
				 	 summary(fit)$coef[(a+p):(a+p-1+indexlast-1),], b.const.remain, c.last ) 		
		}
	  }

	if (apc == 2) {
		b.var = summary(fit,corr=T)$corr[2:a,2:a]
		b.lastse = sqrt(c(matrix(1,1,a-1)%*%diag(summary(fit)$coef[2:a,2])%*%b.var%*%diag(summary(fit)$coef[2:a,2])%*%matrix(1,a-1,1)))
		a.last=c(-sum(fit$coef[2:a]), b.lastse, tt<- -sum(fit$coef[2:a])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

		b.var = summary(fit,corr=T)$corr[-1:-(a+p-2),-1:-(a+p-2)]
		b.lastse = sqrt(c(matrix(1,1,a+p-2)%*%diag(summary(fit)$coef[-1:-(a+p-2),2])%*%b.var%*%diag(summary(fit)$coef[-1:-(a+p-2),2])%*%matrix(1,a+p-2,1)))
		c.last=c(-sum(fit$coef[-1:-(a+p-2)]), b.lastse, tt<- -sum(fit$coef[-1:-(a+p-2)])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

		est <- fit$coef[(a+1):(a+p-2)]
		b.var = summary(fit,corr=T)$corr[(a+1):(a+p-2),(a+1):(a+p-2)]

		####  compute the value for the last constrained estimates
		if ( sum(abs(lindex))>0 && sum(abs(lvalue))>0 ) {

			est.const.last = sum(-lvalue[-nindex] * est[ lindex[-nindex] ])/valuelast   ######## estimate of last constraint
			est = c(est[1:(indexlast - 1)], est.const.last ,  est[-1: - (indexlast - 1)])
			est.last = -sum(est)

			####  compute the variance for the last constrained estimates
		
			b.se.const = summary(fit)$coef[(a+1):(a+p-2),2][lindex[-nindex]] ######## for the se of the param in the constraint ## sum(-lvalue[-nindex]*coh[lindex[-nindex]])/valuelast

			if (nindex ==2)
				b.var.const = lvalue[-nindex]*b.se.const*b.var[lindex[-nindex],lindex[-nindex]]*b.se.const*
				    lvalue[-nindex]/valuelast^2
			else 
				b.var.const = t(as.matrix(lvalue[-nindex]))%*%diag(b.se.const)%*%b.var[lindex[-nindex],lindex[-nindex]]%*%diag(b.se.const)%*%
				    as.matrix(lvalue[-nindex])/valuelast^2			

			index = rep(0, p-2)
			index [lindex[-nindex]] = lvalue[-nindex]
			index.mat = rbind(diag(rep(1,p-2)), index)
			b.lastse = sqrt( sum( index.mat%*% diag(summary(fit)$coef[(a+1):(a+p-2),2]) %*%b.var%*%diag(summary(fit)$coef[(a+1):(a+p-2),2]) %*%t(index.mat) ))
			b.last=c(est.last, b.lastse, tt <- est.last/b.lastse, 2*(1-pt(abs(tt),fit$df)) )

			#### put all parameter estimates and se together 
				
			b.const.remain = c(est.const.last, sqrt(b.var.const), tt<-est.const.last/sqrt(b.var.const), 2*(1-pt(abs(tt),fit$df)) )
			if (nindex < (p-2) )
				b.const.remain = rbind(b.const.remain, summary(fit)$coef[(a+indexlast):(a+p-2),])
			
			b.se = rbind(summary(fit)$coef[1:a,], a.last,  summary(fit)$coef[(a+1):(a+indexlast-1),], b.const.remain, b.last, 
				summary(fit)$coef[-1:-(a+p-2),], c.last) 
		 }
	 }

	if (apc == 1) {
		b.var = summary(fit,corr=T)$corr[a:(a+p-2),a:(a+p-2)]
		b.lastse = sqrt(c(matrix(1,1,p-1)%*%diag(summary(fit)$coef[a:(a+p-2),2])%*%b.var%*%diag(summary(fit)$coef[a:(a+p-2),2])%*%matrix(1,p-1,1)))
		b.last = c(-sum(fit$coef[a:(a+p-2)]), b.lastse, tt<- -sum(fit$coef[a:(a+p-2)])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

		b.var = summary(fit,corr=T)$corr[-1:-(a+p-2),-1:-(a+p-2)]
		b.lastse = sqrt(c(matrix(1,1,a+p-2)%*%diag(summary(fit)$coef[-1:-(a+p-2),2])%*%b.var%*%diag(summary(fit)$coef[-1:-(a+p-2),2])%*%matrix(1,a+p-2,1)))
		c.last = c(-sum(fit$coef[-1:-(a+p-2)]), b.lastse, tt<- -sum(fit$coef[-1:-(a+p-2)])/b.lastse, 2*(1-pt(abs(tt),fit$df)))

		est <- fit$coef[2:(a-1)]
		b.var = summary(fit,corr=T)$corr[2:(a-1),2:(a-1)]

		####  compute the value for the last constrained estimates
		if ( sum(abs(lindex))>0 && sum(abs(lvalue))>0 ) {

			est.const.last = sum(-lvalue[-nindex] * est[ lindex[-nindex] ])/valuelast   ######## estimate of last constraint
			est = c(est[1:(indexlast - 1)], est.const.last ,  est[-1: - (indexlast - 1)])
			est.last = -sum(est)

			####  compute the variance for the last constrained estimates
		
			b.se.const = summary(fit)$coef[2:(a-1),2][lindex[-nindex]] ######## for the se of the param in the constraint ## sum(-lvalue[-nindex]*coh[lindex[-nindex]])/valuelast

			if (nindex ==2)
				b.var.const = lvalue[-nindex]*b.se.const*b.var[lindex[-nindex],lindex[-nindex]]*b.se.const*
				    lvalue[-nindex]/valuelast^2
			else 
				b.var.const = t(as.matrix(lvalue[-nindex]))%*%diag(b.se.const)%*%b.var[lindex[-nindex],lindex[-nindex]]%*%diag(b.se.const)%*%
				    as.matrix(lvalue[-nindex])/valuelast^2

			index = rep(0, a-2)
			index [lindex[-nindex]] = lvalue[-nindex]
			index.mat = rbind(diag(rep(1,a-2)), index)
			b.lastse = sqrt( sum( index.mat%*% diag(summary(fit)$coef[2:(a-1),2]) %*%b.var%*%diag(summary(fit)$coef[2:(a-1),2]) %*%t(index.mat) ))
			a.last=c(est.last, b.lastse, tt <- est.last/b.lastse, 2*(1-pt(abs(tt),fit$df)) )

			#### put all parameter estimates and se together 
				
			b.const.remain = c(est.const.last, sqrt(b.var.const), tt<-est.const.last/sqrt(b.var.const), 2*(1-pt(abs(tt),fit$df)) )
			if (nindex < (a-2) )
				b.const.remain = rbind(b.const.remain, summary(fit)$coef[(1+indexlast):(a-1),])
			
			b.se = rbind(summary(fit)$coef[1:indexlast,], b.const.remain, a.last, 
				summary(fit)$coef[a:(a+p-2),],  b.last, summary(fit)$coef[-1:-(a+p-2),], c.last) 
		 }
	 }
    }				######## Finishing either constrained estimator or intrinsic estimator.

  	if (Plot == T) {
		par(mfrow=c(2,2),cex=pcex)
		if ( abs(pylim[1]-pylim[2]) < 1e-12 )
			plot (age, b.se[1+1:a,1],type='l',ylim=range(cbind(b.se[1+1:a,1],b.se[1+1:a,1]-1.96*b.se[1+1:a,2],b.se[1+1:a,1]+1.96*b.se[1+1:a,2]),na.rm=T),ylab="")
		else 
			plot (age, b.se[1+1:a,1],type='l',ylim=pylim,ylab="")
    
	  points(age, b.se[1+1:a,1])
	  
		if (stderrplot == T) {
			lines(age, b.se[1+1:a,1]-1.96*b.se[1+1:a,2],lty=2)
			lines(age, b.se[1+1:a,1]+1.96*b.se[1+1:a,2],lty=2)
			title(main="Age effect & 95% CI")
		 }
		else 
			title(main="Age effect")


		if ( abs(pylim[1]-pylim[2]) < 1e-12 )
			plot (period, b.se[(a+2):(a+p+1),1],type='l',ylim=range(cbind(b.se[(a+2):(a+p+1),1],b.se[(a+2):(a+p+1),1]-1.96*b.se[(a+2):(a+p+1),2],
				b.se[(a+2):(a+p+1),1]+1.96*b.se[(a+2):(a+p+1),2]),na.rm=T),ylab="")
		else 
			plot (period, b.se[(a+2):(a+p+1),1],type='l',ylim=pylim,ylab="")
	  
	  points(period, b.se[(a+2):(a+p+1),1])

		if (stderrplot == T) {
			lines(period, b.se[(a+2):(a+p+1),1]-1.96*b.se[(a+2):(a+p+1),2],lty=2)
			lines(period, b.se[(a+2):(a+p+1),1]+1.96*b.se[(a+2):(a+p+1),2],lty=2)
			title(main="Period effect & 95% CI")
		 }
		else 
			title(main="Period effect")

		par(mfrow=c(2,1),new=T, fig=c(0,1,0,0.52),cex=pcex)
		if ( abs(pylim[1]-pylim[2]) < 1e-12 )
		  plot (cohort, b.se[-1:-(a+p+1),1], type='l',ylim=range(cbind(b.se[-1:-(a+p+1),1],b.se[-1:-(a+p+1),1]-1.96*b.se[-1:-(a+p+1),2],b.se[-1:-(a+p+1),1]+1.96*b.se[-1:-(a+p+1),2]),na.rm=T),ylab="")	  
		else 
			plot (cohort,  b.se[-1:-(a+p+1),1], type='l',ylim=pylim,ylab="")

		points(cohort, b.se[-1:-(a+p+1),1])

		if (stderrplot == T) {
			lines(cohort, b.se[-1:-(a+p+1),1]-1.96*b.se[-1:-(a+p+1),2],lty=2)
			lines(cohort, b.se[-1:-(a+p+1),1]+1.96*b.se[-1:-(a+p+1),2],lty=2)
			title(main="Cohort effect & 95% CI")
		 }
		else 
			title(main="Cohort effect")

		par(mfrow=c(1,1))
	}
 }

################################################################
######## relabel the variables in output

b.label = dimnames(b.se)[[1]]
b.label[1] = "Intercept"
if (apcmodel == "A") 
	for (i in 1:a) 	b.label[i+1] = paste("Age",as.character(age[i]))

if (apcmodel == "P") 
	for (i in 1:p) 	b.label[i+1] = paste("Period",as.character(period[i]))

if (apcmodel == "C") 
	for (i in 1:(a+p-1)) 	b.label[i+1] = paste("Cohort",as.character(cohort[i]))
 
if (apcmodel == "AP") {
	for (i in 1:a) 	b.label[i+1] = paste("Age",as.character(age[i]))
	for (i in (a+1):(a+p)) b.label[i+1] = paste("Period",as.character(period[i-a]))
 }

if (apcmodel == "AC") {
	for (i in 1:a) 	b.label[i+1] = paste("Age",as.character(age[i]))
	for (i in (a+1):(2*a+p-1)) b.label[i+1] = paste("Cohort",as.character(cohort[i-a]))
 }

if (apcmodel == "PC") {
  for (i in 1:p) 	b.label[i+1] = paste("Period",as.character(period[i]))
  for (i in (p+1):(a+2*p-1)) b.label[i+1] = paste("Cohort",as.character(cohort[i-p]))
}

if (apcmodel == "APC") {
	for (i in 1:a) 	b.label[i+1] = paste("Age",as.character(age[i]))
	for (i in (a+1):(a+p)) b.label[i+1] = paste("Period",as.character(period[i-a]))
	for (i in (a+p+1):(2*a+2*p-1)) b.label[i+1] = paste("Cohort",as.character(cohort[i-a-p]))
 }

dimnames(b.se)[[1]] = b.label
dimnames(b.se)[[2]] = c("Estimate", "Standard.Error","t","P-value")

out = vector("list")
out$model = summary(fit)$call
out$transform = transform
out$varcomp = c(summary(fit)$sig^2, summary(fit)$df[2])
p.val = 2*(1-pf(summary(fit)$f[1],summary(fit)$f[2],summary(fit)$f[3]))
names(p.val) = "p.value"
out$F.stats = c(summary(fit)$f, p.val) 

Rsquared = c(summary(fit)$r.squared, summary(fit)$adj.r.squared)
names(Rsquared) = c("R.squared","  Adjusted.R.Squared")
out$Rsquared = Rsquared
out$parameter = b.se

if (ModelDiag == T) {
  par(mfrow=c(1,1),cex=pcex)
  plot(fit$fitted, fit$resid, xlab="fitted value", ylab="residual")
  abline(a=0,b=0, col=2)
  title(main="Residual Plot for Linear Model Diagnostics")
}

return(out)
}
