apcplot <-
function (r, header=F, scale = "log", P3D = F, apc="age", amin = 1, pmin=1, cmin=1, gapage=1, gapyear=1, 
                    xleg=NA, yleg=NA, nleg=1,pylim=NA, theta=-30, phi=30, p3dcol=3, ppch=F, plty=F, main="3D plot") {

  if (header == T) {
    age = as.numeric(dimnames(r)[[1]])
    per = as.numeric(dimnames(r)[[2]])
    coh = seq(min(per)-max(age), max(per)-min(age),age[2]-age[1])
  }
  else {
    age = amin + (1:(nrow(r)) -1) * gapage 
    per = pmin + (1:(ncol(r)) -1) * gapyear 
    coh = seq(min(per)-max(age), max(per)-min(age), gapage)
  }
  a=nrow(r);   p=ncol(r)
  if (substr(scale,1,3)=="lin" || substr(scale,1,3)=="Lin" || substr(scale,1,2)=="id" || substr(scale,1,3)=="ID") 
    scale = "lin"
  if (P3D == F) {
    if (is.na(sum(pylim)))
      pylim = range(r,na.rm=T)*(scale == "lin") + range(log(r),na.rm=T)*(scale=="log")
    r = log(r)*(scale=="log") + r*(scale=="lin")
    if (substr(apc,1,3) == "age") {
      if (scale == "log") {
        plot (age, r[,1], xlab="age", ylab="log rate", ylim=pylim, type="l")
        title (main = "Age trend by period - log scale")
      }
      if (scale == "lin") {
        plot (age, r[,1], xlab="age", ylab="rate", ylim=pylim, type="l")
        title (main = "Age trend by period - linear scale")
      }
      points (age,r[,1])
      for (i in 2:ncol(r)) {
        if (ppch == F && plty == F) {
          lines(age, r[,i], lty=1, col=i)
          points(age, r[,i], col=i)
        }
        if (ppch == T) {
          lines(age, r[,i], lty=1, col=1)
          points(age, r[,i], pch = as.character(i))
        }
        if (plty == T) {
          lines(age, r[,i], lty=i, col=1)
          points(age, r[,i])
        }
      }
      if (!is.na(xleg+yleg)) {
        if (ppch == F && plty == F)
          legend(x=xleg, y=yleg, lty=1, col=c(1:ncol(r)), legend = paste(as.character(per),"  "), ncol=nleg)
        if (ppch == T)
          legend(x=xleg, y=yleg, lty=1, pch=as.character(c(1:ncol(r))), legend = paste(as.character(per),"  "), ncol=nleg)
        if (plty == T)
          legend(x=xleg, y=yleg, lty=c(1:ncol(r)), legend = paste(as.character(per),"  "), ncol=nleg)
      } 
    }
    
    if (substr(apc,1,3) == "per") {
      if (scale == "log") {
        plot (per, r[1,], xlab="period", ylab="log rate", ylim=pylim, type="l")
        title (main = "Period trend by age - log scale")
      }
      if (scale == "lin") {
        plot (per, r[1,], xlab="period", ylab="rate", ylim=pylim, type="l")
        title (main = "Period trend by age - linear scale")
      }
      points (per,r[1,])
      for (i in 2:nrow(r)) {
#        lines(per, r[i,], lty=1, col=i)
#        points(per, r[i,], col=i)
        if (ppch == F && plty == F) {
          lines(per, r[i,], lty=1, col=i)
          points(per, r[i,], col=i)
          if (!is.na(xleg+yleg)) {
            legend(x=xleg, y=yleg, lty=1, col=c(1:a), legend = paste(as.character(age),"  "),ncol=nleg)
          }
        }
        if (ppch == T) {
          lines(per, r[i,], lty=1, col=1)
          points(per, r[i,], pch = as.character(i))
          if (!is.na(xleg+yleg)) {
            legend(x=xleg, y=yleg, lty=1, pch=as.character(c(1:a)), legend = paste(as.character(age),"  "),ncol=nleg)
          }
        }
        if (plty == T) {
          lines(per, r[i,], lty=i)
          points(per, r[i,])
          if (!is.na(xleg+yleg)) {
            legend(x=xleg, y=yleg, lty=c(1:a), legend = paste(as.character(age),"  "),ncol=nleg)
          }
        }
      }
        
    }
    
    if (substr(apc,1,4) == "coha") {
      if (scale == "lin") {
        plot(coh[1:p], r[a,], xlim=range(coh), ylim=range(r), xlab="birth year", ylab="rate", type="p")
        title (main = "Cohort trend by age - linear scale")
      }
      if (scale == "log") {
        plot(coh[1:p], r[a,], xlim=range(coh), ylim=range(r), xlab="birth year", ylab="log rate", type="p")
        title (main = "Cohort trend by age - log scale")
      }
      
      for (i in 1:a) { 
#        lines(coh[i:(i+p-1)], r[a-i+1,], lty=1, col=i) 
#        points(coh[i:(i+p-1)], r[a-i+1,], col=i) 
      }
      legend(x=xleg, y=yleg, legend=paste(age,"  "), lty=1, col=1:nrow(r), ncol=nleg)
      for (i in 2:nrow(r)) {
        if (ppch == F && plty == F) {
          lines(coh[i:(i+p-1)], r[a-i+1,], lty=1, col=i) 
          points(coh[i:(i+p-1)], r[a-i+1,], col=i) 
          if (!is.na(xleg+yleg))
            legend(x=xleg, y=yleg, legend=paste(age,"  "), lty=1, col=1:nrow(r), ncol=nleg)
        }
        if (ppch == T) {
          lines(coh[i:(i+p-1)], r[a-i+1,], lty=1, col=1) 
          points(coh[i:(i+p-1)], r[a-i+1,], pch=as.character(i%%10)) 
          if (!is.na(xleg+yleg))
            legend(x=xleg, y=yleg, legend=paste(age,"  "), lty=1, pch=as.character((1:nrow(r))%%10), ncol=nleg)
        }
        if (plty == T) {
#         lines(per, r[i,], lty=i)
#          points(per, r[i,])
#          if (!is.na(xleg+yleg)) 
#            legend(x=xleg, y=yleg, lty=c(1:a), legend = paste(as.character(age),"  "),ncol=nleg)
          
          lines(coh[i:(i+p-1)], r[a-i+1,], lty=i, col=1) 
          points(coh[i:(i+p-1)], r[a-i+1,]) 
          if (!is.na(xleg+yleg))
            legend(x=xleg, y=yleg, legend=paste(age,"  "), lty=c(1:nrow(r)), ncol=nleg)
          
        }
      }
      
#      if (!is.na(xleg+yleg)) {
#        legend(x=xleg, y=yleg, lty=1, col=c(a:1), legend = paste(as.character(age), "  "),ncol=nleg)
#      } 
    }
    
    if (substr(apc,1,4) == "cohp") {
      if (scale == "lin") {
        plot(coh[1:a], rev(r[,1]), xlim=range(coh), ylim=range(r), xlab="birth year", ylab="rate", type="p")
        title (main = "Cohort trend by period - linear scale")
      }
      if (scale == "log") {
        plot(coh[1:a], rev(r[,1]), xlim=range(coh), ylim=range(r), xlab="birth year", ylab="log rate", type="p")
        title (main = "Cohort trend by period - log scale")  
      }
      
      for (i in 1:p) { 
        if (ppch == F && plty == F) {
          lines(coh[i:(i+a-1)], rev(r[,i]), lty=1, col=i) 
          points(coh[i:(i+a-1)], rev(r[,i]), col=i) 
          if (!is.na(xleg+yleg))
            legend(x=xleg, y=yleg, lty=1, col=c(1:p), legend = paste(as.character(per), "  "),ncol=nleg)
        }
        if (ppch == T) {
          lines(coh[i:(i+a-1)], rev(r[,i]), lty=1, col=1) 
          points(coh[i:(i+a-1)], rev(r[,i]), , pch=as.character(i))
          if (!is.na(xleg+yleg))
            legend(x=xleg, y=yleg, legend=paste(age,"  "), lty=1, pch=as.character((1:ncol(r))%%10), ncol=nleg)
        }
        if (plty == T) {
          lines(coh[i:(i+a-1)], rev(r[,i]), lty=i, col=1) 
          points(coh[i:(i+a-1)], rev(r[,i]))
          if (!is.na(xleg+yleg))
            legend(x=xleg, y=yleg, legend=paste(age,"  "), lty=c(1:ncol(r)), ncol=nleg)
          
        }
      } 
#      legend(x=xleg, y=yleg, legend=paste(per, "  "), lty=1, col=1:nrow(r), ncol=nleg)
      
    }
    
    
  }
 if (P3D == T) {
   period = per
 if (scale == "lin") { 
     persp(x=age, y=period, z=r, zlab="rate", tick="detailed",
           theta=theta, phi=phi, col=p3dcol)
   }
 if (scale == "log") { 
     r = log(r)
     persp(x=age, y=period, z=r, zlab="log rate", tick="detailed",
           theta=theta, phi=phi, col=p3dcol)
     }
  if (scale == "log")
      title (main = paste(main, "- log scale"))
  if (scale == "lin")
      title (main = paste(main, "- linear scale")) 
  }
}
