\name{apcglmfit}
\alias{apcglmfit}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
APC Generalized Linear Model
}
\description{
Fit a generalized linear model to APC data. So far, loglinear model only for APC data.
}
\usage{
apcglmfit(r, header = F, n.risk = NA, apcmodel = "AP", fam = "loglin", 
Plot = F, pylim = c(0, 0), Scale = 1e-05, apc = 0, lindex = 0, 
lvalue = 0, amin = 1, pmin = 1, cmin = 1, n.interval = 1, pcex = 1)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{r}{ 
  matrix of APC rate or value with rows as age groups and columns as period groups
}
  \item{header}{ 
  logic value, if True, the matrix r has row and column dimnames as header
}
  \item{n.risk}{ 
  population exposure matrix having the same dimension as matrix r. Its header is not required.
}
  \item{apcmodel}{ 
  models to fit APC data: "A", "P", "C" for single factor models; "AP", "AC" for two factor models, or "APC" for full APC model.
}
  \item{fam}{ family of models to fit: "loglin" for loglinear model, or "qlik" for quasi-Poisson (loglinear) model.
}
  \item{Plot}{ 
  logic value, whether to plot the effect estimates
}
  \item{pylim}{ 
  range of values in estimate plot
}
  \item{Scale}{ 
  scales of data r, default is 1e-5 for rate.
}
  \item{apc}{ 
  value to specify factor in a constraint: 1 for age effects; 2 for period effects; 3 for cohort effects. 0 indicates the intrinsic estimator. 
}
  \item{lindex, lvalue}{ 
  values to specify effects and coefficients in a constraint: lindex*lvalue = 0.
}
  \item{amin, pmin, cmin}{ 
  start values of age, period and cohort for labels. Not required for data with header.
}
  \item{n.interval}{ 
  number of units in each age or period group in label. Not required for data with header.
}
  \item{pcex}{ 
  value to specify font of plot charaters.
}
}
\details{ 
Fit a generalized linear model to the frequency following a Poisson distribution with population exposure
as the offset term log(pop). It calculates the frequency with the rates multiplied by the population 
exposure and by the scale as response variable. It estimates fixed effeccts of rows, columns, and diagonals.
It may fit with a quasi-likelihood approach for option fam = "qlik" while default is fam = "loglin". The model may choose to  specify apcmodel = "A", "P", "C" for single factor model, "AP" or "AC" for two factor model, and "APC" for full age-period-cohort model. The full APC model estimates the effects with the intrinsic estimator method (Fu 2016), the standard errors for the period and cohort effects are estimated with the Delta method (Fu 2018). It calls the running horse function apcglmfit_run.
}
\value{
\item{model}{specification of model and family}
        \item{deviance}{model deviance and degrees of freedom}
        \item{pearson.chisq}{the Pearson chi-square statistic}
        \item{p.val}{p-value of the model goodness-of-fit by the deviance following chi-squares distribution.}
        \item{dispersion}{dispersion parameter, if quasi-likelihood is specified}
        \item{parameter}{parameter estimates for the intercept, age, period, and cohort effects, standard errors, Z values (or t values, if quasi-likelihood is specified), and  p-values.}
        }
\references{
Fu, W. (2016) Constrained Estimators and COnsistency of a Regression Model on a Lexis diagram, JASA, Vol 111:180-199. 

Fu, W. (2018) A Practical Guide to Age-Period-COhort Analysis: the Identification Problem and Beyond, CRC. }
\author{
Wenjiang Fu
}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
apclinfit
}
\examples{
x = APCG_data(apcdata="lungCAmale",header=TRUE) 
apcglmfit(r=x$rate,n.risk = x$pop, header=TRUE, Plot=TRUE, 
 apcmodel = "APC",apc=0, Scale = 1e-5,fam="qlik")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }% use one of  RShowDoc("KEYWORDS")
\keyword{ regression }% __ONLY ONE__ keyword per line
