\name{apcplot}
\alias{apcplot}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
APC 2D and 3D Plot 
}
\description{
Plot 2D curves or 3D surface of APC data. 2D curves include age by period, period by age, cohort by age, cohort by period. 
}
\usage{
apcplot(r, header = F, scale = "log", P3D = F, apc = "age", amin = 1, pmin = 1, cmin = 1, gapage = 1, gapyear = 1, xleg = NA, yleg = NA, nleg = 1, pylim = NA, theta = -30, phi = 30, p3dcol = 3, ppch = F, plty = F, main = "3D plot")
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{r}{
Data in a matrix of a by p dimension with a rows (age groups) and p columns (period groups)
}
  \item{header}{
logic value, if True, the matrix dimension labels pass to age and period labels in plots.
}
  \item{scale}{
Scale of data in plot, either the orginal ("id" or "lin") or the log ("log") scale of data. The default is "log".
}
  \item{P3D}{
logic value, if True, produce 3D plot of data with age and period as the axes. 
}
  \item{apc}{
specify effect to plot. "age", "per", "coha" or "cohp" for age by period, period by age, cohort by age, or cohort by period plot, respectively.
}
  \item{amin, pmin, cmin}{
  starting age, period, and cohort labels in plot, respectively, not required if header = T.
}
  \item{gapage, gapyear}{
  number of units (years) in each age or period group.
}
  \item{xleg, yleg}{
  x-, y-coordinates of legend.
  }
  \item{nleg}{
number of columns of legend, default is 1.
}
  \item{pylim}{
range of effect in the plot. Useful when space is needed for legend, either above or below the curves.
}
  \item{theta, phi}{angles defining the viewing direction. theta gives the azimuthal direction and phi the colatitude. 
}
  \item{p3dcol}{
  color of the 3D plot, 0 --- no color, 1 --- black, ...  
}
  \item{ppch}{
  plot character for 2D plot curves.
}
  \item{plty}{
  plot line type for 2D plot curves.
}
  \item{main}{
  title of plot.
}
}
\details{
produce 3D surface plot (P3D = T), or 2D curve plot (P3D = F). 
}
%%\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
%%}
\references{
Fu, W. (2018) A Practical Guide to Age-Period-COhort Analysis: the Identification Problem and Beyond, CRC.
}
\author{
Wenjiang Fu
}
%\note{
%%  ~~further notes~~
%}

%% ~Make other sections like Warning with \section{Warning }{....} ~

%\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
%}
\examples{
USlungCAmale.r = APCG_data(apcdata="lungCAmale",header=TRUE)$rate
apcplot(USlungCAmale.r,header=TRUE, p3d=FALSE, apc="per")
apcplot(USlungCAmale.r,header=TRUE, p3d=FALSE, apc="per", pylim=c(-4, 6))
apcplot(USlungCAmale.r,header=TRUE, p3d=FALSE, apc="per",pylim=c(-4, 6),xleg=1980,yleg=-2.5,nleg=5)
apcplot(USlungCAmale.r,header=TRUE, P3D=TRUE, scale="id", phi=30, theta=30,p3dcol = 3, main ="3D plot of my data")
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ hplot }% use one of  RShowDoc("KEYWORDS")
%\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
